################################################################################################
################################################################################################
################################################################################################
#
#                 This program downloads all selected ice-concentration satellite data
#                         from the pages of the University of Bremen (Germany)
#
#                 https://seaice.uni-bremen.de/sea-ice-concentration/amsre-amsr2/
#                       
#                                (c) Hendrik Pehlke,
#                      Alfred-Wegener-Institute, Bremerhaven, Germany
#
#
#                        The script was written with R version 3.31
#                        and modified with R version 3.6.2 in 2020
#
#       In the timespace from 2002 to 2019 the extent as well as the projection of the raster
#                                        files changed.
#
#           The values of the rasterfiles changed changed also in this timespace:
#
#       First: The maximum value was 255, with values 0-200 for 0 to 100% ice concentration
#              and values > 200 for land and missing values
#       Later: The maximum value was 122, with values 0-100 for 0 to 100% ice concentration
#              and values > 100 for land and missing values
#
#
#                           last changes (dd/mm/yyyy): 09/04/2021
#
################################################################################################
################################################################################################
################################################################################################

#####################################################################################
#
#          install and load needed packages in R-Studio
#
#        !!!!!!  Always execute this part of the script   !!!!!!
#
#####################################################################################
rm(list=ls())  ### Removes everything in the working environment
graphics.off() ### clear all plots
cat("\014")    ### clear console

pckg = c("here",
         "httr",
         "RCurl",
         "svDialogs", 
         "stringr", 
         "tcltk"
)
lapply(pckg, require, character.only = TRUE)
rm(pckg)
##########################################################################################################
##########################################################################################################

here::here()

answer <- "no"
save_path_dir <- c()

while (answer == "no"){
  msgBox("Select now the folder to store the downloaded ice satellite images.")
  
  ### A quick default directory changer
  setwd(dlgDir(default = here(), title="Choose the directory")$res)
  save_path_dir <- getwd()
  
  if (length(save_path_dir) == 1){
    answer <- "yes"
  }
}
rm(answer)


# temp.answer <- "no"
# if (temp.answer =="no") {
#   if (exists("save_path_dir")) { rm(save_path_dir) }
#   save_path_dir <- file.path(choose.dir(default = "C:/", caption = "Select the folder for the downloaded ice raster"))
#   # save_path_dir <- gsub("\\", "", save_path_dir)
# }
# rm(temp.answer)

## Ask a series of items at once in a dialog box
TEXT <- paste(save_path_dir, "/Downloaded_", gsub("-", "_", Sys.Date()), sep="")

tcltk::tkmessageBox(title = "Path to store the data",
             message = paste("The ice concentration raster tifs will be stored in the folder\n\n", TEXT, sep=""), icon = "info", type = "ok")

mainDir <- c(save_path_dir)
subDir <- paste("Downloaded_", gsub("-", "_", Sys.Date()), sep="")

if (file.exists(here(mainDir, subDir))){
  print("Subdirectory already exists!")
} else {
  dir.create(paste0(mainDir, "\\", subDir, sep=""))
}

directory_save_original_ice_data <- paste0(mainDir, "\\", subDir, sep="")

rm(save_path_dir, TEXT, mainDir, subDir)

## URLs to the data
# https://seaice.uni-bremen.de/data/amsre/asi_daygrid_swath/s6250/2002/sep/Antarctic/
# https://seaice.uni-bremen.de/data/amsr2/asi_daygrid_swath/s6250/2012/aug/Antarctic/

url_a = c('https://seaice.uni-bremen.de/data/amsre/asi_daygrid_swath/s6250')
url_b = c('https://seaice.uni-bremen.de/data/amsr2/asi_daygrid_swath/s6250')

# File name and thus also the name of the respective 'Child URL'.
# The exact specification by means of year and month is done in the 
# for loop
filename1 =("asi-s6250-")  # for years 2002 - 2011
filename2 = ("asi-AMSR2-s6250-") # for years 2012 - till now
# Ending of the file names
ending = ("-v5.4.tif")

location <- "Antarctic"
month <- c("jan",
           "feb",
           "mar",
           "apr",
           "may",
           "jun",
           "jul",
           "aug",
           "sep",
           "oct",
           "nov",
           "dec")
month_nr <- c("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12")
days <- c("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16",
          "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31")

####### Select the polar region
answer <- "no"
region <- c()

while (answer == "no"){
  dlgMessage("Please choose the polar region (Arctic or Antarctic)")$res
  while (length(region)==0){
    region <- select.list(c("Arctic", "Antarctic"), 
                                  preselect = NULL, 
                                  title = "Select one polar region",
                                  multiple = FALSE, 
                                  graphics = TRUE)
  }
  if (length(region) == 1){
    answer <- "yes"
  }
}
rm(answer)

if (region == "Arctic") {
  # https://seaice.uni-bremen.de/data/amsre/asi_daygrid_swath/n6250/
  filename1 =("asi-n6250-")  # for years 2002 - 2011
  filename2 = ("asi-AMSR2-n6250-") # for years 2012 - till now
  location <- "Arctic"
  url_a = c('https://seaice.uni-bremen.de/data/amsre/asi_daygrid_swath/n6250')
  url_b = c('https://seaice.uni-bremen.de/data/amsr2/asi_daygrid_swath/n6250')
}

#######  now choose the years
actual.year <- as.numeric(format(Sys.time(), "%Y"))
years <- as.character(seq(2002, actual.year))

answer <- "no"
while (answer == "no"){
  dlgMessage("Please choose the year(s) you want to download")$res
  
  ## Select the year(s) you want to download
  year_selection <- c()
  while (length(year_selection) < 1) {
    #year_selection <- dlgList(years, preselect = c("2002"), multiple = TRUE, graphics=TRUE)$res
    year_selection <- select.list(years, 
                                  preselect = c("2002"), 
                                  title = "Select one or more years",
                                  multiple = TRUE, 
                                  graphics = TRUE)
  }
  if (length(year_selection>=1)) {
    answer <- "yes"
  }
}
# rm(years, answer)
########


#######  now choose the months
answer <- "no"
while (answer =="no"){
  dlgMessage("Please select the month(s):")$res
  ## Select the year(s) you want to download
  month_selection <- c()
  while (length(month_selection) < 1) {
    #year_selection <- dlgList(years, preselect = c("2002"), multiple = TRUE, graphics=TRUE)$res
    month_selection <- select.list(month, 
                                   preselect = c("jan",
                                                 "feb",
                                                 "mar",
                                                 "apr",
                                                 "may",
                                                 "jun",
                                                 "jul",
                                                 "aug",
                                                 "sep",
                                                 "oct",
                                                 "nov",
                                                 "dec"), 
                                   title = "Select the month(s)",
                                   multiple = TRUE, 
                                   graphics = TRUE)
  }
  if (length(month_selection>=1)){
    answer <- "yes"
  }
}
# rm(month, answer)
########

# handle years with different satellites resp. devices
year_selection_1 <- c()
year_selection_2 <- c()

year_selection_1 <- subset(year_selection, year_selection %in% c("2002", "2003", "2004", "2005", "2006", "2007", "2008", "2009", "2010", "2011"))
year_selection_2 <- subset(year_selection, year_selection %in% as.character(seq(2012, actual.year)))

rm(year_selection)

#################################################################################################################################
# https://seaice.uni-bremen.de/data/amsre/asi_daygrid_swath/s6250/2010/jan/Antarctic/asi-s6250-20100101-v5.4.tif

urls_months <- c()
urls_days <- c()

### if years from 2002 - 2011 are chosen
y=1
cat("\014") # clear console
if (length(year_selection_1) > 0) {
  for (y in 1:length(year_selection_1)){
    m=1
    for (m in 1:length(month_selection)){
      url1 <- paste(url_a, year_selection_1[y], month_selection[m], location, sep="/")
      urls_months <- c(urls_months, url1)
    
      ttest <- c()
      ttest <-as.numeric(httr::GET(url1)[2])
      
      if (ttest != 404){
        print(url1)
        print("URL exists")
        # } # end of 'if (ttest != 404)'
        
        thepage <- c()
        thepage = suppressWarnings(readLines(url1, ok = TRUE, warn = FALSE, skipNul = TRUE)) # read all line from the url1
        
        #----
        if (length(thepage) >=1){  
          # Find the lines that contain the names for tif files
          nc4.lines <- c()
          nc4.lines <- grep(".tif", thepage)  # select from the lines just the lines containing '.tif'
          # Subset the original dataset leaving only those lines
          thepage <- thepage[nc4.lines] 
          rm(nc4.lines)
          
          #extract the row numbers with the file names
          if (exists("str.loc")){ rm(str.loc) }
          str.loc <- str_locate(thepage,'asi.*v5.4.tif?"')
          
          # create a list with the file names with means of 'substring'
          file.list <- c()
          file.list <- substring(thepage,str.loc[,1], str.loc[,2]-1)
          file.list <- na.omit(file.list)  # remove NA if exists
          
          for ( ifile in file.list){
            url = paste0(url1, "/", noquote(ifile), sep="")
            urls_days <- c(urls_days, url)
            rm(url)
          }
        }
      }
    }
  }
}
#################################################################################################################################
### if years from 2011 are chosen

### example url
# https://seaice.uni-bremen.de/data/amsr2/asi_daygrid_swath/s6250/2017/jan/Antarctic/asi-AMSR2-s6250-20170101-v5.tif

if (length(year_selection_2) > 0) {
  
  ### for the years after 2011
  y=1
  for (y in 1:length(year_selection_2)){
    m=1
    for (m in 1:length(month_selection)){
      url1 <- paste(url_b, year_selection_2[y], month_selection[m], location, sep="/")
      
      ttest <- c()
      ttest <-as.numeric(httr::GET(url1)[2])
      
      if (ttest != 404){
        print(url1)
        print("URL exists")
        # } # end of 'if (ttest != 404)'
        
        thepage <- c()
        thepage = readLines(url1) # read all line from the url1
        
        #----
        if (length(thepage) >=1){  
          # Find the lines that contain the names for tif files
          nc4.lines <- c()
          nc4.lines <- grep(".tif", thepage)  # select from the lines just the lines containing '.tif'
          # Subset the original dataset leaving only those lines
          thepage <- thepage[nc4.lines] 
          rm(nc4.lines)
          
          #extract the row numbers with the file names
          if (exists("str.loc")){ rm(str.loc) }
          str.loc <- str_locate(thepage,'asi.*v5.4.tif?"')
          
          # create a list with the file names with means of 'substring'
          file.list <- c()
          file.list <- substring(thepage,str.loc[,1], str.loc[,2]-1)
          file.list <- na.omit(file.list)  # remove NA if exists
          for ( ifile in file.list){
            url = paste0(url1, "/", noquote(ifile), sep="")
            urls_days <- c(urls_days, url)
            rm(url)
          }
        }
      }
    }
  }
}


# Download all selected tif-files
#----------------------------------------------------------------------------------------------------
i <- 1          
for ( i in 1:length(urls_days)){
  print(paste(i, " of total ", length(urls_days), sep=""))
  file.name <- unlist(strsplit(urls_days[i],"/"))[length(unlist(strsplit(urls_days[i],"/")))]
  # print(file.name)
  GET(urls_days[i], 
      write_disk(path=paste(directory_save_original_ice_data, file.name, sep = "\\"), overwrite = TRUE)
      )
  rm(file.name)
  
}


##----------------------------------------------------------------------------------------------------------------------------------
tkmessageBox(title = "R E M I N D E R",
             message = paste0("The downloaded ice concentration files are stored in the folder:\n ", directory_save_original_ice_data), icon = "info", type = "ok")

# rm(list=setdiff(ls(), c("directory_save_original_ice_data", "month", "years")))
# gc()

