################################################################################################
################################################################################################
################################################################################################
#
#                 Check downloaded ice-concentration data. Does a file
#                                exists for each date?
#                       
#                                (c) Hendrik Pehlke
#                   Alfred-Wegener-Institute, Bremerhaven, Germany
#
#                       Source for sea ice concentration tifs: 
#           https://seaice.uni-bremen.de/sea-ice-concentration/amsre-amsr2/
#
##
#         This R-script checks for all days beginning the 01.01.2002 until now if an
#             ice-concentration tif-file is available in the path you define.
#
#     In the resulting table, a "1" indicates that a tif-file exists, an "NA" indicates that
#                        there is no tif-file for that date.
#
#           The resulting table will be stored under the name you choose
#
#                     The sript was written with R version 3.3.1
#
#                      last changes (dd/mm/yyyy): 09/04/2021
#
#
################################################################################################
################################################################################################
################################################################################################

#####################################################################################
#
#          install and load needed packages in R-Studio
#
#        !!!!!!  Always execute this program part   !!!!!!
#
#####################################################################################

####################################################################################
####        Step 1: Load needed packages, define function used later            ####
####################################################################################
library(pacman)

# define the packages that will be installed and loaded
pacman::p_load("here", 
               "openxlsx",
               "svDialogs", 
               "tcltk2"
               )
p_loaded()
#####################################################################################
rm(list=ls())  ### Removes everything in the working environment
graphics.off() ### clear all plots
cat("\014")    ### clear console
gc()

#####################################################################################
#
#     end of installing and loading of all packages listed in 'pckg'
#
#####################################################################################

# define path
here::here()

answer <- "no"
save_path_tiffs <- c()

while (answer == "no"){
  msgBox("Where are the the downloaded ice satellite images (geo-tiffs)?")
  
  ### A quick default directory changer
  setwd(dlgDir(default = here(), title="Choose the directory with downloaded ice satellite images.")$res)
  save_path_tiffs <- getwd()
  
  if (length(save_path_tiffs) == 1){
    answer <- "yes"
  }
}
rm(answer)

answer <- "no"
save_path_table <- c()

while (answer == "no"){
  msgBox("Choose the directory to store the resulting availability table (*.csv)")
  
  setwd(dlgDir(default = here(), title="Choose the directory to store the resulting avaiability table (*.csv)")$res)
  save_path_table <- getwd()
  if (length(save_path_table)==1) {
    answer <- "yes"
  }
}
rm(answer)

setwd(save_path_tiffs)
# make a list of all downloaded tif files in the selected folder
ice_list <- list.files(pattern="tif$")


actual.year <- as.numeric(format(Sys.time(), "%Y"))

year <- as.character(seq(from=2002, to = actual.year, by = 1))
month <- c("01","02","03","04","05","06","07","08","09","10","11","12")
days <- c("01","02","03","04","05","06","07","08","09","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31")

rm(actual.year)

## fill search pattern with the combination of year and (number of) months and create a list of these
## search patterns
suchpattern <- c()

for (a in 1:length(year)) {
  for (b in 1:length(month)) {
    for (c in 1:length(days)) {
      suchname <- c(paste(year[a], month[b], days[c], sep=""))
      #print(suchname)
      suchpattern <- rbind(suchpattern, suchname)
    }
  }
}
rm(a,b,c, suchname)
suchpattern <- sort(as.character(suchpattern[,1]))

setwd(save_path_tiffs)

## create empty list
files <- c()

####################################################################
## function number of days in month
numberOfDays <- function(date) {
  m <- format(date, format="%m")
  
  while (format(date, format="%m") == m) {
    date <- date + 1
  }
  
  return(as.integer(format(date - 1, format="%d")))
}
####################################################################

if(exists("result")) { rm(result) }

# create an empty table (data frame)
result <- as.data.frame(matrix(nrow = (length(year)*length(month)), ncol = 33, byrow = TRUE,
                               dimnames = list(c(),
                                               c("year","month",c(days)))))
result$sum <- NA
result$max <- NA
result$diff <- NA
n=1
c <- 1

for (c in 1:length(year)){
  #print(paste("Monat: ",month[a]))
  #result[n,1] <- year[c]
  a <- 1
  for (a in 1:length(month)){
    #print(paste("Tag: ", days[b]))
    #liste <- c()
    files <- c()
    result[n,1] <- year[c]
    result[n,2] <- month[a]
    
    v =3
    b <- 1
    for (b in 1:length(days)) {
      print(paste("Jahr: ", year[c],", Monat: ",month[a], ", Tag: ", days[b], sep=""))
      test <- as.character(paste(year[c],month[a],days[b], "-v5.4.tif",sep=""))
      
      date = as.Date(paste(year[c], month[a], "01", sep="-"), "%Y-%m-%d")  # create data
      result$max[n] <- numberOfDays(date)
      
      setwd(save_path_tiffs)
      # print(test)
      file <- list.files(getwd(), pattern=test, recursive=F, full.names=F)
      #print(file)
      if (length(file)>0){
        print("Y E S  ! ! !")
        result[n,v] <- 1
        print("________________________")
      }
      rm(test, file)
      v=v+1
    }
    n=n+1
  }
}

result$sum <- rowSums (result[c(3:33)], na.rm = TRUE)
result$diff <- result$max-result$sum

View(result)

rm(a,b,c, days, files, ice_list, month, n, suchpattern, v, year)

setwd(save_path_table)

save.name <- paste("Overview_of__the_availability_of_daily_ice_concentration_files_2002_to_", format(Sys.time(), "%Y"), "_status_", format(Sys.time(), "%Y-%m-%d"), ".csv", sep="")
write.table(result, file = save.name, sep = ",", qmethod = "double")
rm(save.name)

save.name <- paste("Overview_of__the_availability_of_daily_ice_concentration_files_2002_to_", format(Sys.time(), "%Y"), "_status_", format(Sys.time(), "%Y-%m-%d"), ".xlsx", sep="")
write.xlsx(result, file = save.name, keepNA = TRUE, colNames = TRUE, borders = "surrounding", colWidths = "auto", overwrite = TRUE)

rm(result, save_path_table, save_path_tiffs)
