################################################################################################
################################################################################################
################################################################################################
#
#                   Analyse the metadata of the downloaded raster files
#              e.g. projection, number of cols, number of rows, resolution etc
#                       
#                              (c) Hendrik Pehlke
#                   Alfred-Wegener-Institute, Bremerhaven, Germany
#
#                         Source for sea ice concentration tifs: 
#           https://seaice.uni-bremen.de/sea-ice-concentration/amsre-amsr2/
#
#
#                     The sript was written with R version 3.3.1
#
#                      last changes (dd/mm/yyyy): 09/04/2021
#
#
################################################################################################
################################################################################################
################################################################################################
library(pacman)

# define the packages that will be installed and loaded
pacman::p_load("here", 
               "gdalUtils",
               "raster", 
               "rgdal", 
               "svDialogs")
p_loaded()

#####################################################################################
rm(list=ls())  ### Removes everything in the working environment
graphics.off() ### clear all plots
cat("\014")    ### clear console
gc()

## define path to the downloaded ice concentration tifs
here::here()

answer <- "no"
save_path_tiffs <- c()

while (answer == "no"){
  msgBox("Where are the the downloaded ice satellite images (geo-tiffs)?")
  
  ### A quick default directory changer
  setwd(dlgDir(default = here(), title="Choose the directory with downloaded ice satellite images.")$res)
  save_path_tiffs <- getwd()
  
  if (length(save_path_tiffs) == 1){
    answer <- "yes"
  }
}
rm(answer)

setwd(save_path_tiffs)
getwd()

# Create a list of all available tif files in the selected folder
tif.liste <- dir(pattern=".tif")

## Create a new empty data frame to store the results of the analyses.
if(exists("tiff.analyse")) { rm(tiff.analyse) }
tiff.analyse <- data.frame(matrix(ncol=13,
                                  nrow=0, 
                                  dimnames=list(NULL, c("date", 
                                                        "tif_name", 
                                                        "projection_name",
                                                        "EPSG_nr",
                                                        "projection_prj4",
                                                        "lower_left_origin_X",
                                                        "lower_left_origin_Y",
                                                        "count_row",
                                                        "count_col",
                                                        "res_x",
                                                        "res_y",
                                                        "min_value",
                                                        "max_value"))))


## loop over all tifs to analyse the raster files (=metadata)
i <- 1
for (i in 1:length(tif.liste)){
  print(paste(i, " of total ", length(tif.liste)))
  if (exists("tif.info")) { rm(tif.info) }
  tif.info <- capture.output( rgdal::GDALinfo(tif.liste[i]) )
  cat("\014") # clear console
  #tif.info
  
  if(exists("t.temp")) {rm(t.temp)}
  t.temp <- unlist(strsplit((unlist(strsplit(tif.info[13], " "))[length(unlist(strsplit(tif.info[13], " ")))]), "-"))
  t.temp <- unlist(t.temp)[length(unlist(t.temp))-1]
  as.Date(t.temp, "%Y%m%d")
  tiff.analyse[c((nrow(tiff.analyse))+1), "date"] <- as.character(as.Date(t.temp, "%Y%m%d"))
  rm(t.temp)
  tiff.analyse[c(nrow(tiff.analyse)), "tif_name"] <- unlist(strsplit(tif.info[13], " "))[length(unlist(strsplit(tif.info[13], " ")))]
  if(c(gsub("projection ", "", tif.info[12])) != " +proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 +y_0=0 +a=6378273 +b=6356889.449 +units=m +no_defs ") {
    tiff.analyse[c(nrow(tiff.analyse)), "projection_name"] <- "unknown"
    }
  if(c(gsub("projection ", "", tif.info[12])) == " +proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 +y_0=0 +a=6378273 +b=6356889.449 +units=m +no_defs ") {
    tiff.analyse[c(nrow(tiff.analyse)), "projection_name"] <- "NSIDC Sea Ice Polar Stereographic South"
    }
  if(c(gsub("projection ", "", tif.info[12])) == " +proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs ") {
    tiff.analyse[c(nrow(tiff.analyse)), "projection_name"] <- "WGS 84 / NSIDC Sea Ice Polar Stereographic South"
    }
  if(c(gsub("projection ", "", tif.info[12])) != " +proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 +y_0=0 +a=6378273 +b=6356889.449 +units=m +no_defs ") {
    tiff.analyse[c(nrow(tiff.analyse)), "EPSG_nr"] <- "unknown"
    }
  if(c(gsub("projection ", "", tif.info[12])) == " +proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 +y_0=0 +a=6378273 +b=6356889.449 +units=m +no_defs ") {
    tiff.analyse[c(nrow(tiff.analyse)), "EPSG_nr"] <- "3412"
    }
  if(c(gsub("projection ", "", tif.info[12])) == " +proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs ") {
    tiff.analyse[c(nrow(tiff.analyse)), "EPSG_nr"] <- "3413"
    }
  
  tiff.analyse[c(nrow(tiff.analyse)), "projection_prj4"] <- c(gsub("projection ", "", tif.info[12]))
  tiff.analyse[c(nrow(tiff.analyse)), "lower_left_origin_X"] <- as.integer(unlist(strsplit(tif.info[4], " "))[length(unlist(strsplit(tif.info[4], " ")))])
  tiff.analyse[c(nrow(tiff.analyse)), "lower_left_origin_Y"] <- as.integer(unlist(strsplit(tif.info[5], " "))[length(unlist(strsplit(tif.info[5], " ")))])
  tiff.analyse[c(nrow(tiff.analyse)), "count_row"] <- as.integer(unlist(strsplit(tif.info[1], " "))[length(unlist(strsplit(tif.info[1], " ")))])
  tiff.analyse[c(nrow(tiff.analyse)), "count_col"] <- as.integer(unlist(strsplit(tif.info[2], " "))[length(unlist(strsplit(tif.info[2], " ")))])
  tiff.analyse[c(nrow(tiff.analyse)), "res_x"] <- as.integer(unlist(strsplit(tif.info[6], " "))[length(unlist(strsplit(tif.info[6], " ")))])
  tiff.analyse[c(nrow(tiff.analyse)), "res_y"] <- as.integer(unlist(strsplit(tif.info[7], " "))[length(unlist(strsplit(tif.info[7], " ")))])
  tiff.analyse[c(nrow(tiff.analyse)), "min_value"] <- suppressWarnings(min(na.omit(as.integer(unlist(strsplit(tif.info[19], " ")))), na.rm = FALSE))
  tiff.analyse[c(nrow(tiff.analyse)), "max_value"] <- suppressWarnings(max(na.omit(as.integer(unlist(strsplit(tif.info[19], " ")))), na.rm = FALSE))
}

unique(tiff.analyse$projection_prj4)
rm(tif.liste, i, tif.info)

# Select directory to store the result table in
answer <- "no"
save_path_table <- c()

while (answer == "no"){
  msgBox("Choose the directory to store the resulting availability table (*.csv)")
  
  setwd(dlgDir(default = here(), title="Choose the directory to store the resulting metadata table (*.csv)")$res)
  save_path_table <- getwd()
  if (length(save_path_table)==1) {
    answer <- "yes"
  }
}
rm(answer)

setwd(save_path_table)
write.csv(tiff.analyse, "Downloaded_ice_raster_files_metadata.csv", sep="")
