################################################################################################
################################################################################################
################################################################################################
#                 Check downloaded ice-concentration data. Does a file
#                                exists for each date?
#                       
#                           (c) Hendrik Pehlke, June 2017
#                   Alfred-Wegener-Institute, Bremerhaven, Germany
#                      
#                        Source for sea ice concentration tifs: 
#           https://seaice.uni-bremen.de/sea-ice-concentration/amsre-amsr2/
#
#
#             Analyse every single downloaded ice concentration tiff.
#          Examine each grid cell: finde range and other values over all
#                          stacked raster data for each cell
#
#              This script calculate accessibility (or the navigability)
#            based on the ice concentrations and a chosen threshold value
#
#            It calculates the Repeated Accessibility (RA) value for each 
#      individual day and raster cell for a time period selected by the user 
#      (2, 3 or 4 years). This percentage value is the calculated probability
#       that this grid cell will be reachable (navigable) at least twice 
#         within the selected time period at the chosen day of the year.
#                       
#                             
#                    last changes (dd/mm/yyyy): 09/04/2021
#
#
################################################################################################
################################################################################################
################################################################################################

library(pacman); pacman::p_unload("all")

rm(list=ls()) ### Removes everything in the working environment
dev.off()     ### Remove all plots
cat("\014") # clear console

#####################################################################################
#
#          install and load needed packages in R-Studio
#
#        !!!!!!  Diesen Programmteil immer ausf?hren   !!!!!!
#
#####################################################################################
library(pacman)
# define the packages that will be installed and loaded
pacman::p_load("imputeTS", 
               "psych",
               "raster",
               "rgdal",
               "sp",
               "svDialogs",
               "tools", 
               "utils", 
               "zoo")
p_loaded()

rm(list=ls()) ### Removes everything in the working environment
graphics.off()     ### Remove all plots
cat("\014") # clear console
gc()

#####################################################################################
#
#     end of installing and loading of all packages listed in 'pckg'
#
#####################################################################################

#####  create functions

####
# create function 1: calculate the number of days of a month
# numberOfDays <- function(date) {
#   m <- format(date, format="%m")
#   
#   while (format(date, format="%m") == m) {
#     date <- date + 1
#   }
#   
#   return(as.integer(format(date - 1, format="%d")))
# }

####
# create function 2: clip raster data with shape

# clip<-function(raster,shape) {
#  a1_crop<-crop(raster,shape)
#  step1<-rasterize(shape,a1_crop)
#  a1_crop*step1}

# clear console
#cat("\014") # clear console

####
# create function 3: remove words from string
# removeWords <- function(str, dropwords) {
#   x <- unlist(strsplit(str, " "))
#   paste(x[!x %in% dropwords], collapse = " ")
# }

####
# create round function >=0.5 to 1, <0.5 to 0
# rnd <- function(x) trunc(x+sign(x)*0.5)
# 
# ####
# # create function 4: replace NAs with mean of all other columns
# f.mean <- function(df,g,h){
#   m <- rowMeans(df[,g:h], na.rm = TRUE)
#   #m <- mean(x, na.rm = TRUE)
#   df[is.na(df)] <- rnd(m)
#   df
# } 

################################################################################################################
################################################################################################################
################################################################################################################
#
#   Programmteil 1: Auswahl der Daten (Jahre und Monate)
#
#
################################################################################################################
################################################################################################################
################################################################################################################

##### define path to original data and path to store result data

# Ort der Originaldaten bestimmen (z.B. "Z:/02_raw_data/gis/raster/sea_ice/bremen/02_data_bremen")
# C:\temp\bremen_ice_satellite_tifs

answer <- "no"#
data.select <- c()
while (answer == "no"){
  svDialogs::msgBox("Please select one of the downloaded ice cover tifs.")
  data.select <- file.choose(getwd())
  if (length(data.select) ==1){
    answer <- "yes"
  }
} # end of 'answer <- "no"'
rm(answer)

ice_data <- dirname(data.select)
ice_data <- as.character(ice_data)

new.proj <- raster::projection(raster::raster(data.select))

setwd(ice_data)
tif.liste <- dir(pattern=".tif$")

cat("\014") # clear console
head(tif.liste, 50)
# tail(tif.liste, 70)

## detect years and months of the downloaded ice data
year_list <- c()
month_list <- c()
a <- 1
for (a in 1:length(tif.liste)){
  if (length(unlist(strsplit(tif.liste[a], "-"))) == 4){
    tttt <- unlist(strsplit(tif.liste[a], "-"))[3]
  }
  if (length(unlist(strsplit(tif.liste[a], "-"))) == 5){
    tttt <- unlist(strsplit(tif.liste[a], "-"))[4]
  }  
  tt_y <- as.numeric(substr(tttt, start = 1, stop = 4))
  tt_m <- substr(tttt, start = 5, stop = 6)
  
  year_list <- c(year_list, tt_y)
  month_list <- c(month_list, tt_m)
  rm(tttt, tt_y, tt_m)
  
}
year_list <- sort(unique(year_list))
month_list <- sort(unique(month_list))

rm(a, data.select)

################################################################################
## select the path to store the result data
answer <- "no"
storage_path <- c()
while (answer == "no"){
  svDialogs::msgBox("Please select the directory in which the results are to be saved")
  setwd(choose.dir(caption = "directory to save the data"))
  storage_path <- as.character(getwd())
  if (length(storage_path == 1)){
    answer <- "yes"
  }
} # end of 'while (answer == "no"){'
rm(answer)

#######################################
## Do you want to clip the raster files with a polygon shape. This is recommended
answer <- "no"
while (answer == "no"){
  clip_answer <- svDialogs::dlgMessage("Do you want to limit the area? \nCircumpolar calculations can take an extremely long time or even cause the programme to crash.",
                                       "yesno")$res
  if (clip_answer == "no") {
    answer <- "yes"
  }
  if (clip_answer == "yes") {
    answer2 <- "no"
    shape_to_clip <- c()
    while (answer2 == "no"){
      svDialogs::msgBox("Please select the polygon shapefile (*.shp) to be used for clipping")
      shape_to_clip <-rgdal::readOGR(utils::choose.files(default = getwd(),
                                                         caption = "Select shapefile",
                                                         multi = FALSE) )
      if (length(shape_to_clip)==1){
        answer2 <- "yes"
        answer <- "yes"
      }
    } # end of 'while (answer2 == "no"){'
  } # end of 'if (clip_answer == "yes") {'
} # end of 'while (answer == "no"){'
rm(answer, clip_answer, answer2)


# Here the projection of the raster data is assigned to the shape file
shape_to_clip <- sp::spTransform(shape_to_clip, 
                                 # CRS("+proj=stere +lat_0=-90 +lat_ts=-70 +lon_0=0 +k=1 +x_0=0 +y_0=0 +a=6378273 +b=6356889.449 +units=m +no_defs")
                                 CRS(new.proj)
)

# plot(shape_to_clip)
setwd(storage_path)
save.image(paste("IMAGE_RA_Analyse_001_", format(Sys.time(), "%b_%d_%Y_%H"),"_oclock", ".RData", sep=""))

################################################################################
cat("\014") # clear console
## Monatswahl: the months to analyse
# years <- as.character(seq(2002, as.numeric(format(Sys.Date(), "%Y")), by = 1))
years <- as.character(year_list)
# month <- c("01","02","03","04","05","06","07","08","09","10","11","12")
month <- as.character(month_list)
days <- c("01","02","03","04","05","06","07","08","09","10","11","12","13","14","15",
          "16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31")

################################################################################
## now select the month(s) you want to analyse
answer <- "no"
Monatswahl <- c()
while (answer =="no"){
  svDialogs::dlgMessage("Please select the month(s) you would like to examine:")$res
  
  ## Select a month
  Monatswahl <- select.list(choices = month, 
                            preselect = month[12], 
                            multiple = TRUE,
                            title = "Select month(s)", 
                            graphics = TRUE)
  
  
  Monatswahl <- sprintf("%02d", match(Monatswahl, month))
  if (length(Monatswahl)>=1){
    answer <- "yes"
  }
}

################################################################################
answer <- "yes"
while(answer == "yes"){
  answer2 <- svDialogs::dlgMessage("Would you like to select specific years?", "yesno")$res
  while (answer2 == "yes"){
    year_list <- select.list(choices = years, 
                             preselect = c(years), 
                             multiple = TRUE,
                             title = "Select year(s)", 
                             graphics = TRUE)
    if (length(year) >= 1){
      answer2 <- "no"
    }
  }
  if (answer2 == "no"){
    answer <- "no"
  }
}
rm (answer, answer2)

if (exists("year_list")) { 
  year <- year_list
  rm(year_list) 
}


################################################################################
answer <- "yes"
while(answer == "yes"){
  answer2 <- svDialogs::dlgMessage("Would you like to select specific days?", "yesno")$res
  while (answer2 == "yes"){
    day_list <- select.list(choices = days, 
                            preselect = c(days), 
                            multiple = TRUE,
                            title = "Select year(s)", 
                            graphics = TRUE)
    if (length(day_list) >= 1){
      answer2 <- "no"
    }
  }
  if (answer2 == "no"){
    answer <- "no"
  }
}
rm(answer, answer2)

if (exists("day_list")) { 
  days <- day_list
  rm(day_list) 
}

################################################################################
## define the threshold for accessibility
## values (in a grid cell) above this threshold stands for 'full ice cover'

# Accessibility to grid cell i in year j at day x is given by
# Ai,j,x = 0 or 1									(1)
# where sea ice cover > 60 % is defined as 0 (inaccessible) and sea ice cover <= 60 % is defined
# as 1 (accessible) according to Parker et al. (2014, Methodology for automated spatial sea ice
# summaries in the Southern Ocean, CCAMLR Science, WG-FSA-14/54: "Although more than 85% of 
# fishing events occurred in areas with less than 20% sea ice, a threshold of 40-60% sea ice 
# indicated the transition from fished to non-fished conditions.", "(.) a threshold range of 
# 40-60% sea ice concentration is a useful proxy to describe an area as fishable.")

answer <- "no"
threshold <- c()
while (answer == "no"){
  svDialogs::msgBox("Please define the maximum value of the ice concentration with regard to navigability")
  tttt  <- "no"
  while (tttt == "no"){
    threshold <- as.numeric(svDialogs::dlgInput("Ice coverage value of the transition from fished to non-fished conditions. According to Parker (2014): '40-60% sea ice concentration is a proxy to describe an area as fishable'. \nSelect a value between 0 and 100.", default = "60")$res)
    if (threshold < 0){
      svDialogs::msgBox("The value you have selected is too small. \nPlease select a value between 0 and 100.")
      #threshold <- c()
    }
    if (threshold > 100) {
      svDialogs::msgBox("The value you have selected is too large. \nPlease select a value between 0 and 100.")
      #threshold <- c()
    }
    if (threshold >= 0 & threshold <= 100){
      tttt <- "yes"
    }
  } # end of 'while (tttt == "no"){'
  if (length(threshold ==1)){
    answer <- "yes"
  }
} # end of 'while (answer == "no"){'
rm(answer, tttt)


################################################################################
# Define the maximum time period (in years) in which a repeated scientific investigation must take place.
answer <- "no"
my <- c()
while (answer =="no"){
  svDialogs::dlgMessage("Please define the maximum time span (in years) in which a repeated scientific investigation must take place:")$res
  my <- as.numeric(svDialogs::dlgInput("Please define the maximum time span (<= 4 years) in which a repeated scientific investigation must take place (CCAMLR guidelines: 2 years.):", default = "2")$res)
  if (length(my) == 0) {
    msgBox("No time span was selected. \nTherefore, the maximum time span was set to 2 years.")
    my <- 2
  } # end of 'if (length(my) == 0)'
  
  if (my >= 5) {
    svDialogs::msgBox("A period of time has been chosen which is >= 5 years.\nTherefore, the maximum time span was set to 4 years.")
    my <- 4
  } # end of 'if (length(my) == 0)'
  if (length(my)==1){
    answer <- "yes"
  }
}


setwd(storage_path)
save.image(paste("IMAGE_RA_Analyse_002_", format(Sys.time(), "%b_%d_%Y_%H"),"_oclock", ".RData", sep=""))
cat("\014") # clear console

# alte langsame Version  
#######################################################################################################################
#######################################################################################################################
#######################################################################################################################
setwd(ice_data)
# a stands for months, b stands for days and c stands for years
a = b = c = 1

for (a in 1:length(Monatswahl)){  # end of 'for (a in 1:length(Monatswahl))'
  #print(paste("Monat: ",Monatswahl[a]))
  for (b in 1:length(days)) {
    #print(paste("Tag: ", days[b]))
    if(exists(as.character(substitute(liste)))){rm(liste)}   # delete file if exists
    liste <- c()
    for (c in 1:length(year)){
      print(paste("############# Jahr: ", year[c],", Monat: ", Monatswahl[a], ", Tag: ", days[b], sep=""))
      
      if(exists(as.character(substitute(test)))){rm(test)}   # delete file if exists
      if (exists("test")) { rm(test) }
      test <- as.character(paste(year[c],Monatswahl[a],days[b],sep=""))
      liste <- c(liste, test)
      liste <- sort(liste)
      rm(test)
      ## create empty list
      files <- c()
      setwd(ice_data)
      x <- 1
      for (x in 1:length(liste)) {
        #print(liste[x])
        file <- list.files(getwd(), pattern=liste[x], recursive=F, full.names=F)
        file <- as.character(file)
        files <- c(files, file)
      } # end of 'for (x in 1:length(liste))'
    } # end of 'for (c in 1:length(year))'
    
    rm(x)
    
    # now stack all selected raster and clip with them shape then reclass the raster
    #rm(r)
    print(paste("Verarbeite Raster 1 von ", length(files), ". Name of raster: ", c(tools::file_path_sans_ext(files[1])), sep=""))
    
    if(exists(as.character(substitute(r)))){rm(r)}   # delete file if exists
    r <- raster::raster(files[1])
    
    # if(psych::describe(raster::as.data.frame(r))$max<120) {
    #   ok_cancel_box("!!! Maximum value of a grid is greater than 255 !!!!")
    # }
    if(psych::describe(raster::as.data.frame(r))$max<120) {
      ok_cancel_box("!!! Maximum value of a grid is greater than 255 !!!!")
    }
    
    max.V <- psych::describe(raster::as.data.frame(r))$max
    
    names(r) <- c(tools::file_path_sans_ext(files[1]))
    
    r <- raster::crop(r, shape_to_clip)
    r <- raster::mask(r, shape_to_clip, inverse=FALSE, updatevalue = max.V)
    rm(max.V)
    
    d <- 2
    for (d in 2:length(files)){
      print(paste("Verarbeite Raster ",d, " von ", length(files),  ", name: ", c(tools::file_path_sans_ext(files[d])),sep=""))
      
      if(exists(as.character(substitute(r2)))){rm(r2)}   # delete file if exists
      r2 <- raster::raster(files[d])
      
      if(psych::describe(raster::as.data.frame(r2))$max<120) {
        ok_cancel_box("!!! Maximum value of a grid is greater than 255 !!!!")
      }
      
      max.V <- psych::describe(raster::as.data.frame(r2))$max
      
      r2 <- raster::crop(r2, shape_to_clip)
      r2 <- raster::mask(r2, shape_to_clip, inverse=FALSE, updatevalue = max.V)
      rm(max.V)
      
      
      r <- raster::stack(r,r2)
      
      rm(r2)
    } # end of 'for (d in 2:length(files))'
    
    raster::nlayers(r)
    
    #if(exists(as.character(substitute(x)))){rm(x)}   # delete file if exists
    stopwords = c("asi", "s6250", "AMSR2", "s6250", "v5.4")
    
    x <- unlist(strsplit(c(tools::file_path_sans_ext(files)), "-"))
    
    x <- x[!x %in% stopwords]
    names(r) <- paste("YMD_", c(x), sep="")
    
    
    if(exists(as.character(substitute(test2)))){rm(test2)}   # delete file if exists
    test2 <- c(raster::maxValue(r))
    test2 <- unique((test2 >100))
    
    if (test2) {
      r <- raster::reclassify(r, c(100.0001, Inf, NA))
    } # end of 'if (test2)'
    rm(test2)
    
    # names(r) <- paste("YMD_", c(x), sep="")
    
    setwd(storage_path)
    
    if(exists(as.character(substitute(name_year)))){rm(name_year)}   # delete file if exists
    name_year <- as.numeric(substring(x, 1, 4))
    
    # Abspeichen des neuen Rasters mit neuen Namen 
    name <- c(paste("rasterstack_month_",Monatswahl[a],"_day_",days[b],"_years_", min(name_year), "_to_",max(name_year),".tif", sep=""))
    # writeRaster(r, filename = name, bylayer=FALSE, format="GTiff", overwrite=TRUE)
    
    
    ###############################################################################################################
    ### reclassify raster stack to values 1 or 0
    #
    # reclassify the raster values into two classes (0 = not accessible, 1= assessible)
    # using the threshold the user defined
    if(exists(as.character(substitute(r_reclass)))){rm(r_reclass)}   # delete file if exists
    r_reclass <- raster::reclassify(r, c(-Inf,threshold,1, threshold,Inf,0))
    
    #plot(r_reclass, maxnl=raster::nlayers(r_reclass))
    
    name_year <- as.numeric(substring(x, 1, 4))
    
    # Abspeichen des neuen Rasters mit neuen Namen 
    name <- c(paste("rasterstack_month_",Monatswahl[a],"_day_",days[b],"_years_", min(name_year), "_to_",max(name_year),"_ice_threshold_accessibility_reclass.tif", sep=""))
    # writeRaster(r_reclass, filename = name, bylayer=FALSE, format="GTiff", overwrite=TRUE)
    
    rm(name)
    # rm(r)
    # r <- r_reclass
    
    ###############################################################################
    #  RA   WIEDERERREICHBARKEIT
    ###############################################
    
    # Calculate the new cell values from 0 to max. 4 in the different years.
    
    if(exists(as.character(substitute(df.ice)))){rm(df.ice)}   # delete file if exists
    
    ### replace NAs with rolling mean, here mean of a 3 x 3 window
    if(exists("r.focal")) { rm(r.focal) }
    r.focal <- r
    for (i in 1:raster::nlayers(r.focal)){
      print(i)
      r.focal[[i]] <- raster::focal(r.focal[[i]], w=matrix(1,nrow=3, ncol=3), fun=mean, NAonly=TRUE, na.rm=TRUE)
    }
    
    names(r.focal) <- c(names(r))
    if(exists("df.ice")) { rm(df.ice) }
    df.ice <- raster::as.data.frame(r.focal, row.names=NULL, xy=TRUE)
    
    if(exists(as.character(substitute(select_test)))){rm(select_test)}   # delete file if exists
    select_test <- df.ice[,1:2]
    
    
    # Hier wird berprft ob Daten fr die Kombination Tag und Jahr vorliegen.
    # Sollte dieses nicht der Fall sein, dann wird der Datensatz auf den Wert
    # 1 gesetzt (A = 1 = erreichbar)
    #
    # ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG
    #
    # Hier ist noch nicht das Problem mit dem Monat Dezember gelst. In diesem
    # Monat fehlt das letzte Jahr 2016. In dem folgenden Run wird hier noch der
    # Wert 1 vergeben.
    
    f <- 1
    #c.abs <- 0
    for (f in 1:length(liste)){
      print(liste[f])
      
      ## check if column for year exist
      if (length(df.ice[,grep(liste[f], names(df.ice))]) > 0) {
        #print("Yes")
        select_test$n <- c(df.ice[,grep(liste[f], colnames(df.ice), value=TRUE)])
        colnames(select_test)[length(names(select_test))]<- c(liste[f])
      } # end of 'if (length(df.ice[,grep(liste[f], names(df.ice))]) > 0)'
      
      # if column for year does not exists, create it and fill it with NA
      if (length(df.ice[,grep(liste[f], names(df.ice))]) == 0) {
        print("No")
        #c.abs <- c.abs+1
        select_test$n <- NA
        colnames(select_test)[length(names(select_test))]<- c(liste[f])
      } # end of 'if (length(df.ice[,grep(liste[f], names(df.ice))]) == 0)'
    } # end of 'for (f in 1:length(liste))'
    rm(f)
    #########
    
    sum_na_before <- sum(is.na(select_test[c(3:ncol(select_test))]))
    
    
    if(exists(as.character(substitute(df)))){rm(df)}   # delete file if exists
    df <- select_test[3:ncol(select_test)]
    col.nr <- ncol(df)
    # df[c(59690:59700),]
    # Missing Value Imputation by Weighted Moving Average (package: imputeTS)
    start.time <- Sys.time()
    # library(imputeTS)
    # y <- t(as.matrix(df[c(59690:59700),]))
    y <- t(as.matrix(df))
    
    options(warn = -1)
    
    ############################################################################
    ##          Missing Value Imputation by Weighted Moving Average
    # Missing value replacement by weighted moving average. 
    # Uses semi-adaptive window size to ensure all NAs are replaced
    # here with:
    # k = 1, integer width of the moving average window. Expands to both sides of the center element e.g. k=2 means 4 observations (2 left, 2 right)
    # weighting = simple, Simple Moving Average (SMA)
    # maxgap = 5, Maximum number of successive NAs to still perform imputation on.
    x1 <- imputeTS::na_ma(y, k = 1, weighting = "simple", maxgap = 5)
    x1 <- t(x1)
    options(warn = 0)
    stop.time <- Sys.time()
    y <- t(y)
    
    #View(x1[c(59690:59700),])
    # y[c(59690:59700),]
    
    # now combine the results with the x and y coordinates
    select_test <- cbind(select_test[,c(1:2)], x1)
    # select_test [c(59690:59700),]
    sum_na_after <- sum(is.na(select_test[c(3:ncol(select_test))]))
    # number of replaced NAs
    sum_na_before-sum_na_after
    
    ##########################################################
    if (exists("df.test")) {rm(df.test)}
    if (exists("df.access")) {rm(df.access)}
    df.test <-select_test[,c(3:ncol(select_test))]
    df.access <- apply(df.test, 1, function(x) {ifelse((x <= threshold), 1, 0)})
    df.access <- as.data.frame(t(df.access))
    
    # df.access[c(59690:59700),]
    # View(df.test[(59690:59700),])
    ##########################################################
    
    rm(y, x1, x2, i, df.test)
    
    select_test <- cbind(select_test[,c(1:2)], df.access)
    
    ########
    if (exists("tt")) {rm(tt)}
    tt <- select_test
    coordinates(tt) <- ~x+y
    gridded(tt) <- TRUE
    
    test.result <- raster::stack(tt)
    raster::crs(test.result) <- new.proj
    
    # change the projection to the projection of the shape ("South_Pole_Stereographic", EPSG 102020)
    # +proj=laea +lat_0=-90 +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs
    # projection(test.result) <- CRS("+proj=laea +lat_0=-90 +lon_0=0 +x_0=0 +y_0=0 +datum=WGS84 +units=m +no_defs")
    # plot(test.result, , maxnl=raster::nlayers(test.result))
    
    name <- c(paste(storage_path,"/", "Accessibility_rasterstack_month_",Monatswahl[a],"_day_",days[b],"_years_", min(year), "_to_",max(year),"_threshold_",threshold,"_each_year.tif", sep=""))
    raster::writeRaster(test.result, filename = name, options="INTERLEAVE=BAND", overwrite=TRUE)
  #} # ??? wo
  # rm(name, tt, test.result)
  
  
  ##  RA CALCULATION
  if (exists("df.ice")) {rm(df.ice)}
  df.ice <- select_test
  #rm(select_test, f)
  
  # Now calculate if "Wiedererreichbarkeit" is given
  df.ice$rowSum <- 0
  df.ice$rowSum <- rowSums(df.ice[,3:ncol(df.ice)])
  
  #########################################################################################################################
  ### TEST RA calculation
  if (exists("test_df")) {rm(test_df)}
  if (exists("df.RA_values")) {rm(df.RA_values)}
  
  # test.df <- select_test[58942,c(3:ncol(select_test))]
  test.df <- select_test[,c(3:ncol(select_test))]
  
  ##############################################################################
  ###      Apply Rolling Functions
  # A generic function for applying a function to rolling margins of an array.
  df.RA_values <- abs(t(apply(test.df, 1, function(x) {
    ifelse(x == 1, 
           zoo::rollapplyr(x, c(my+1), 
                           sum, 
                           align = "right")-x, 0)
  })))
  summary(df.RA_values)
  ########
  
  if (exists("data_safe")) {rm(data_safe)}
  data_safe <- as.data.frame(df.RA_values)
  
  
  data_safe$rowSum <- NA
  data_safe$rowSum <- rowSums(data_safe[1:(ncol(data_safe)-1)])
  data_safe$percent <- ( c(data_safe$rowSum/my) / (ncol(data_safe)-1) )*100
  
  data_ra_given <- data_safe[1:c(ncol(data_safe)-2)]
  #data_ra_given <- apply(data_ra_given, 1, function(x) {ifelse((x <= 1), 1, 0)})
  data_ra_given[data_ra_given >= 1] <- 1
  data_ra_given$rowSum <- NA
  data_ra_given$rowSum <- rowSums(data_ra_given[1:(ncol(data_ra_given)-1)])
  data_ra_given$percent <- ( c(data_ra_given$rowSum) / (ncol(data_ra_given)-1) )*100
  
  if(exists(as.character(substitute(gg)))){rm(gg)}   # delete file if exists
  gg <- df.ice[1:2]
  gg <- cbind(gg, data_safe$percent)
  
  coordinates(gg) <- ~x+y
  gridded(gg) <- TRUE
  
  if(exists(as.character(substitute(r.result)))){rm(r.result)}   # delete file if exists
  r.result <- raster::raster(gg)
  raster::crs(r.result) <- new.proj
  
  plot(r.result, main=paste("Quality of Repeated Accessibility within ", my," years", sep=""))
  Sys.sleep(0.5)
  
  ##############################################################################
  gg2 <- df.ice[1:2]
  gg2 <- cbind(gg2, data_ra_given$percent)
  
  coordinates(gg2) <- ~x+y
  gridded(gg2) <- TRUE
  
  if(exists(as.character(substitute(r.result2)))){rm(r.result2)}   # delete file if exists
  r.result2 <- raster::raster(gg2)
  raster::crs(r.result2) <- new.proj
  
  plot(r.result2, main=paste("Existence of Repeated Accessibility within ", my," years", sep=""))
  Sys.sleep(0.5)
  #################################################################################################
  # now safe the percent RA raster grids
  #################################################################################################
  
  
  # Within a time period of e.g. 3 years the maximum cell value of RA is 3 (every year accessibility is given).
  # If you examine data of 10 years the summed maximum value could be 30 (10 times 3). Then 30 would be 100%.
  # Imagine now in 7 of the 10 years the maximum value of 3 is given, but in three years there are lower RA
  # values: 0 (no repeated RA), 1 (one RA is given) and 2 (in two of three cases a RA is given).
  # The summed RA of this example would be 24 ((7*3)+2+1+0). The percentage value would be 80%.
  name <- c(paste(storage_path,"/", "RA_quality_month_",Monatswahl[a],"_day_",days[b],"_years_", min(year), "_to_",max(year),"_threshold_",threshold,"_time_span_1_year_plus_",my,"_years_result_in_percent.tif", sep=""))
  raster::writeRaster(r.result, filename = name, format="GTiff", overwrite=TRUE)
  rm(name, r.result, liste, file, files, data_safe)
  rm(gg)
  gc()
  
  #######################
  # Here all RA value >=1 were set to 1, meaning alt least one RA was given in the time span, but 
  # the absolute frequency is ignored. If you examine data of 10 years, the possible overall maximum
  # would be 10 (100%)
  # Imagine now in 7 of the 10 years the maximum value of 3 is given, but in three years there are lower RA
  # values: 0 (no repeated RA), 1 (one RA is given) and 2 (in two of three cases a RA is given).
  # Now all values except of the 0 are set to 1. The result is 9 ((7*1)+1+1+0). The percentage value would
  # be 90%.
  name <- c(paste(storage_path,"/", "RA_exits_at_least_once_month_",Monatswahl[a],"_day_",days[b],"_years_", min(year), "_to_",max(year),"_threshold_",threshold,"_time_span_1_year_plus_",my,"_years_result_in_percent.tif", sep=""))
  raster::writeRaster(r.result2, filename = name, format="GTiff", overwrite=TRUE)
  rm(r.result2)
  rm(gg2)
  gc()
  
  
} # end of 'for (b in 1:length(days))'
} # end of 'for (a in 1:length(Monatswahl))'


rm(a,b,c)
svDialogs::msgBox("Programmende")
